 
/**
* Clase Consola
* @author Silvia Takahashi
*/

package uniandes.teolen.automatas.interfaseProbador;


import javax.swing.*;

import uniandes.teolen.automatas.mundoAutomata.*;

import java.awt.*;
import java.awt.event.*;
import java.lang.reflect.InvocationTargetException;
import java.util.*;



public class ConsolaAutomatas extends JPanel{

	
	
    // Los autmatas que son manejados por esta consola
	
	private static MundoAutomatas mundo;
	
	/**
	 * El area de texto que se usa para mostrar que automata se esta usando
	 */
	static private JLabel title =  new JLabel();
	
	//The Vector of commands for history
  	static Vector <String> commands = new Vector<String> ();

  	//The number of commands
  	static int contCom = 0;	
	
	/**
	 * El area de texto que se usa para los mensajes del sistema
	 */
	static private JTextArea sistema =  new JTextArea(10,50);
	
	/**
	 * El campo de texto en el que el usuario puede escribir mensajes para que
	 * la consola los interprete
	 */
	static private JTextArea orden = new JTextArea(2,50);
	
	/**
	* Boton para enviar texto al interprete
	*/
	static private JButton enviar = new JButton("Enviar");
	
	/**
	* Boton para Salir
	*/
	static private JButton salir = new JButton("Salir");

	/** 
	* ScrollPanes para que  el log y la entrada se pueda mostrar con ScrollBars
	*/ 
        
	static private JScrollPane scrollPaneSistema = new JScrollPane(sistema);
	static private JScrollPane scrollPaneOrden = new JScrollPane(orden);

	/**
	 * Runnable class para controlar el flujo de ejecucin al escribir en la
	 * ventana del sistema
	 */
	
	final OutputPrinter escribirEnSistema = new OutputPrinter();
	
	/*  Dialogo */
	private OneIntegerDialog moduloQue =new OneIntegerDialog("Contador Modulo N","Qu es N?",3);
	
	/* Menu */
	private JMenuBar mb;

	
	static private ConsolaAutomatas consola;
	static private JFrame frame;
	/**
	 * Crea una nueva consola que se encarga de dar ordenes a un determinado
	 * mundo directamente o a traves de un Interprete
	 * @param mundo El mundo al que esta consola manejara
	 */
	
	
	
	
	
	public ConsolaAutomatas(MundoAutomatas ma) {
		
		mundo = ma;
	    
		orden.addKeyListener(new KeyAdapter(this));
		
	    agregarAcciones();
	    
	    
		
		//El log no se debe poder modificar
		sistema.setEditable(false);
		//title.setEditable(false);
		title.setText(mundo.getCurrentAuto().toString());
		title.setHorizontalAlignment(SwingConstants.CENTER);
		/*
		 * El GridBagLayout es otra forma de poner los componentes, mucho mas
		 * flexible, que permite decidir como son las posiciones, dimensiones y
		 * comportamientos de los objetos unos con otros
		 */
        GridBagLayout gridBag = new GridBagLayout();
        setLayout(gridBag);
        GridBagConstraints c = new GridBagConstraints();
		//Hace que el componente ocupe todo lo ancho
        c.gridwidth = GridBagConstraints.REMAINDER;
        c.fill = GridBagConstraints.BOTH;
		gridBag.setConstraints(scrollPaneSistema,c);
        
		c.weightx =1.0;
        c.weighty = 1.0;
        gridBag.setConstraints(title,c);
        
        add(title);
        c.fill = GridBagConstraints.HORIZONTAL;
        gridBag.setConstraints(scrollPaneSistema,c);
        add(scrollPaneSistema);
        gridBag.setConstraints(scrollPaneOrden,c);
        add(scrollPaneOrden);
        add(enviar);
     	add(salir);
     	initMenu();
     	frame.setJMenuBar(mb);
     	
   }
	
public static void relaunch(int dim){
		
		frame.pack();
		
	    //Center the window
		Dimension frameSize = frame.getSize();
	    frame.setResizable(true);
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		
		if (frameSize.height > screenSize.height) {
			frameSize.height = screenSize.height;
		}
		if (frameSize.width > screenSize.width) {
			frameSize.width = screenSize.width;
		}
		frame.setLocation((screenSize.width - frameSize.width) / 2, (screenSize.height - frameSize.height) / 2);
		frame.setVisible(true);
	
 }	 
	
	/**
	 * Ejecuta sincrnicamente la orden de escrbir en TextArea sistema
	 * @param msg
	 */
	
	private void imprimirSalida(String msg) {
	    
		escribirEnSistema.setMessage(msg);
		try {
			SwingUtilities.invokeAndWait(escribirEnSistema);
		} catch (InterruptedException e) {
					e.printStackTrace();
		} catch (InvocationTargetException e) {
					e.printStackTrace();
		}
	
	}
	
	/**
	 * Agrega los mtodos para los dos botones de salir y Ejecutar
	 */
	private void agregarAcciones() {
		salir.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
						System.exit(0);
					}
			 });
			 
		enviar.addActionListener(new ActionListener() {
		
			public void actionPerformed(ActionEvent e) {
					
				final Thread doInterpretar = new Thread() {
					public void run() {
						String texto;
						String resp;
		
						// Automata currentAuto = mundo.getCurrentAuto();
						
						orden.setEditable(false);
						enviar.setEnabled(false);
						imprimirSalida("Usuario: \n" + orden.getText()+"\n");
						texto=orden.getText();
						orden.setText("");
						commands.addElement(texto);
						contCom = commands.size();
						
						resp = mundo.procesarCadena(texto);
						/*
						currentAuto.setInput(texto);
						try {
							if (currentAuto.processString())
								resp = "Auto "+ currentAuto+ " Acepto la cadena: "+texto;
							else
								resp = "Auto "+ currentAuto + " No Acepto la cadena: "+texto;
							resp= resp + "\n Respuesta: " + currentAuto.getOutput();		
						}
						catch (Exception e)
						{
							resp = "ERROR "+ currentAuto;
							e.printStackTrace() ;
						}
						*/
						orden.setEditable(true);
						orden.setEnabled(true);
						enviar.setEnabled(true);
						imprimirSalida("\n" + resp +"\n");
					}
				};
				doInterpretar.start();
			}
		}); 
	}

	void orden_keyPressed(KeyEvent e) {
		int keyCode = e.getKeyCode();

	    //up
		if (keyCode==38){
			if (contCom>0){
				orden.setText( (commands.elementAt(contCom-1)).toString() );
				contCom=contCom-1;
			}
		}

		//down
		if (keyCode==40){
			if (contCom < commands.size()-1 ){
				orden.setText( (commands.elementAt(contCom+1)).toString() );
				contCom=contCom+1;
			}
		}

	}
	
	private void setAutomata(Automata a) {
		mundo.setCurrentAuto(a);
		title.setText(a.toString());
		title.repaint();
	}
	
	
	private void initMenu() {
		
		
		
		mb = new JMenuBar();
  		
  		JMenu menuAutomata = new JMenu("Automata");
  		JMenu menuAyuda = new JMenu("Ayuda");
  		JMenu menuAutomata1 = new JMenu("Automatas Parametrizados");
  		
  			
  		 for (int i = 0 ; i < mundo.sizeAutos(); i++) {
 			final Automata a = mundo.getAuto(i);
 			JMenuItem item =  new JMenuItem(a.toString());		
 			
 			item.addActionListener(new ActionListener() {
 				public void actionPerformed(ActionEvent e) {
 					setAutomata(a);
 				}
 			});
 			
 			menuAutomata.add(item);
 			
 		}
    
        ;
        for (int i = 0; i < mundo.sizeAutosP() ; i++) {
        	final int auto = i; 
        	JMenuItem itemP =  new JMenuItem(mundo.getAutoP(i));
        	 
        	itemP.addActionListener(new ActionListener() {
      			  public void actionPerformed(ActionEvent e) {
      					moduloQue.setVisible(true);
      					if (moduloQue.hasInput()) {
      						int f = moduloQue.getInput();
      						setAutomata(mundo.getAutoP(auto,f));
      					}
      				}
        		 }
      		   );
        	   
        	  menuAutomata1.add(itemP);
             
    	} 
  	    ;
        
        
       
       JMenuItem itemAbout = new JMenuItem("Acerca De...");
		

	
		mb.add(menuAutomata);
		mb.add(menuAutomata1);
		mb.add(menuAyuda);
		
				

		
		menuAyuda.add(itemAbout);
		itemAbout.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				MessageDialog m = new MessageDialog();
				m.showMessage("About...","Probador de Autmatas");
			}
		});
		
        


		
		
		
		frame.setJMenuBar(mb);
	}

	
	
	/**
	 * Crea un control de tipo Consola con su correspondiente visualizacion
	 * Para ello debe crear un Frame, ubicarlo en la pantalla y hacerlo
	 * visible. 
	 * 	 */

	public static void main(String args[]) {
		//Si no hay exactamente un argumento o no es un numero debe mandar un
		//mensaje

		
		
		try {
	    if (args.length != 0) {
			throw new Exception("No Debe haber  argumentos");
	    }
	   
	    }
		catch(Exception ex) {
			System.exit(1);
		}
	
		frame = new JFrame("Probador de Autmatas");
		
		/* Hace que la aplicacin salga cuando se cierra esta ventana*/
		frame.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                System.exit(0);
            }  
        });
		
		
		consola = new ConsolaAutomatas(new MundoAutomatas());
		
		//Hace que lo que hay en el archivo se pueda mover con barras
		frame.getContentPane().setLayout(new BorderLayout());
		frame.getContentPane().add(consola);
		frame.setSize(100, 100);
		frame.pack();
		
	    //Center the window
		Dimension frameSize = frame.getSize();
		frame.setResizable(true);
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		
		if (frameSize.height > screenSize.height) {
			frameSize.height = screenSize.height;
		}
		if (frameSize.width > screenSize.width) {
			frameSize.width = screenSize.width;
		}
		frame.setLocation((screenSize.width - frameSize.width) / 2, (screenSize.height - frameSize.height) / 2);
		frame.setVisible(true);
	 }

	
	
/**
 * 
 * @author Silvia Takahashi
 *
 * Clase para poder ejecutar sincrnicamente los cambios en el 
 * Panel de sistema
 * 
 */
class OutputPrinter implements Runnable {

	String message;
	
	public OutputPrinter() {
	}
	public void run() {
				sistema.append(message);		
	 }
	 public void setMessage(String s) {
	 	message = s;
	 }
}

class KeyAdapter extends java.awt.event.KeyAdapter {
  
  ConsolaAutomatas adaptee;

  KeyAdapter(ConsolaAutomatas adaptee) {
    this.adaptee = adaptee;
  } 
  
  public void keyPressed(KeyEvent e) {
    adaptee.orden_keyPressed(e);
  }
 
}
}
